@@ -49,6 +49,7 @@ public:
           indexReader( 0 ),
           indexWriter( 0 ),
           analyzer( 0 ),
+          queryAnalyzer( 0 ),
           searcher( 0 ),
           deleteAnalyzer( false ),
           transactionID( 0 ) {
@@ -58,6 +59,7 @@ public:
     lucene::index::IndexReader* indexReader;
     lucene::index::IndexWriter* indexWriter;
     lucene::analysis::Analyzer* analyzer;
+    lucene::analysis::Analyzer* queryAnalyzer;
     lucene::search::IndexSearcher* searcher;
 
     bool deleteAnalyzer;
@@ -181,11 +183,14 @@ public:
         int cnt = 0;
         lucene::document::DocumentFieldEnumeration* it = doc->fields();
         while ( it->hasMoreElements() ) {
-            it->nextElement();
-            ++cnt;
+            lucene::document::Field* field = it->nextElement();
+            TString fieldName( field->name(), true );
+            if ( fieldName != Index::idFieldName() &&
+                 fieldName != Index::textFieldName() )
+                ++cnt;
         }
         delete it;
-        return cnt <= 2;
+        return cnt == 0;
     }
 
     lucene::document::Document* getDocument( const Node& resource ) {
@@ -218,8 +223,8 @@ public:
                         docWrapper.addProperty( field->name(), field->stringValue(), !field->isIndexed() );
                     }
                 }
-                delete( fields );
-                delete( oldDoc );
+                _CLDELETE( fields );
+                _CLDELETE( oldDoc );
             }
 
             // step 4: add the new doc to our cache
@@ -253,7 +258,7 @@ public:
             if ( !docEmpty( doc ) ) {
                 getIndexWriter()->addDocument( doc );
             }
-            delete( doc );
+            _CLDELETE( doc );
         }
 
         documentCache.clear();
@@ -285,7 +290,7 @@ Soprano::Index::CLuceneIndex::~CLuceneIndex()
 
 bool Soprano::Index::CLuceneIndex::open( const QString& folder, bool force )
 {
-    qDebug() << "CLuceneIndex::open in thread " << QThread::currentThreadId();
+//    qDebug() << "CLuceneIndex::open in thread " << QThread::currentThreadId();
     close();
 
     clearError();
@@ -320,7 +325,7 @@ bool Soprano::Index::CLuceneIndex::open( const QString& folder, bool force )
         }
     }
 
-    qDebug() << "CLuceneIndex::open done in thread " << QThread::currentThreadId();
+//    qDebug() << "CLuceneIndex::open done in thread " << QThread::currentThreadId();
 
     return true;
 }
@@ -328,7 +333,7 @@ bool Soprano::Index::CLuceneIndex::open( const QString& folder, bool force )
 
 void Soprano::Index::CLuceneIndex::close()
 {
-    qDebug() << "CLuceneIndex::close in thread " << QThread::currentThreadId();
+//    qDebug() << "CLuceneIndex::close in thread " << QThread::currentThreadId();
     clearError();
     if ( d->transactionID ) {
         closeTransaction( d->transactionID );
@@ -336,7 +341,7 @@ void Soprano::Index::CLuceneIndex::close()
     QMutexLocker lock( &d->mutex );
     d->closeReader();
     d->closeWriter();
-    qDebug() << "CLuceneIndex::close done in thread " << QThread::currentThreadId();
+//    qDebug() << "CLuceneIndex::close done in thread " << QThread::currentThreadId();
 }
 
 
@@ -347,26 +352,34 @@ bool Soprano::Index::CLuceneIndex::isOpen() const
 }
 
 
+void Soprano::Index::CLuceneIndex::setQueryAnalyzer( lucene::analysis::Analyzer* analyzer )
+{
+    d->queryAnalyzer = analyzer;
+}
+
+
+lucene::analysis::Analyzer* Soprano::Index::CLuceneIndex::queryAnalyzer() const
+{
+    return d->queryAnalyzer;
+}
+
+
 lucene::document::Document* Soprano::Index::CLuceneIndex::documentForResource( const Node& resource )
 {
-    qDebug() << "CLuceneIndex::documentForResource in thread " << QThread::currentThreadId();
     QMutexLocker lock( &d->mutex );
 
     clearError();
 
     if ( d->transactionID == 0 ) {
         setError( "No transaction started." );
-        qDebug() << "CLuceneIndex::documentForResource done in thread " << QThread::currentThreadId();
         return 0;
     }
     try {
-        qDebug() << "CLuceneIndex::documentForResource done in thread " << QThread::currentThreadId();
         return d->getDocument( resource );
     }
     catch( CLuceneError& err ) {
         qDebug() << "(Soprano::Index::CLuceneIndex) Exception occured: " << err.what();
         setError( exceptionToError( err ) );
-        qDebug() << "CLuceneIndex::documentForResource done in thread " << QThread::currentThreadId();
         return 0;
     }
 }
@@ -513,14 +526,11 @@ Soprano::Error::ErrorCode Soprano::Index::CLuceneIndex::removeStatement( const S
 
 Soprano::Node Soprano::Index::CLuceneIndex::getResource( lucene::document::Document* document )
 {
-    qDebug() << "CLuceneIndex::getResource in thread " << QThread::currentThreadId();
     QString id = TString( document->get( idFieldName().data() ) );
     if ( id.startsWith( bnodeIdPrefix() ) ) {
-        qDebug() << "CLuceneIndex::getResource done in thread " << QThread::currentThreadId();
         return Soprano::Node( id.mid( bnodeIdPrefix().length() ) );
     }
     else {
-        qDebug() << "CLuceneIndex::getResource done in thread " << QThread::currentThreadId();
         return Soprano::Node( QUrl( id ) );
     }
 }
@@ -530,13 +540,20 @@ Soprano::Iterator<Soprano::Index::QueryHit> Soprano::Index::CLuceneIndex::search
 {
     clearError();
     try {
-        lucene::search::Query* q = lucene::queryParser::QueryParser::parse( TString( query ).data(), textFieldName().data(), d->analyzer );
-        Iterator<QueryHit> hits = search( q );
-        // FIXME: is it possible to use the stupid CLucene ref counting here?
-        if ( !hits.isValid() ) {
-            delete q;
+        lucene::queryParser::QueryParser parser( textFieldName().data(), d->queryAnalyzer ? d->queryAnalyzer : d->analyzer );
+        lucene::search::Query* q = parser.parse( TString( query ).data() );
+        if ( !q ) {
+            setError( QString( "Failed to parse CLucene query: '%1'" ).arg( query ) );
+            return Iterator<QueryHit>();
+        }
+        else {
+            Iterator<QueryHit> hits = search( q );
+            // FIXME: is it possible to use the CLucene ref counting here?
+            if ( !hits.isValid() ) {
+                delete q;
+            }
+            return hits;
         }
-        return hits;
     }
     catch( CLuceneError& err ) {
         qDebug() << "search" << query << "failed: " << err.what();
@@ -687,7 +704,7 @@ void Soprano::Index::CLuceneIndex::clear()
 {
     if ( d->indexPresent() ) {
         try {
-            int numDocs = d->getIndexReader()->numDocs();
+            int numDocs = d->getIndexReader()->maxDoc();
             for ( int i = 0; i < numDocs; ++i ) {
                 d->getIndexReader()->deleteDocument( i );
             }
